package com.ichaoj.ams.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ichaoj.ams.entity.AmsAirdropProject;
import com.ichaoj.ams.entity.AmsAirdropTask;
import com.ichaoj.ams.entity.AmsExecuteRecord;
import com.ichaoj.ams.mapper.AmsExecuteRecordMapper;
import com.ichaoj.ams.request.execute.CreateExecute;
import com.ichaoj.ams.request.execute.PageExecuteRequest;
import com.ichaoj.ams.request.execute.UpdateExecute;
import com.ichaoj.ams.response.execute.ExecuteResponse;
import com.ichaoj.ams.service.IAmsAirdropProjectService;
import com.ichaoj.ams.service.IAmsAirdropTaskService;
import com.ichaoj.ams.service.IAmsExecuteRecordService;
import com.ichaoj.common.exception.ErrorServiceException;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicUserInfo;
import com.ichaoj.mybatis.service.SuperWhaleServiceImpl;
import com.ichaoj.web.context.SuperWhaleContext;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * <p>
 * 自动化 服务实现类
 * </p>
 *
 * @author cjwen
 * @since 2023-05-18
 */
@Service
public class AmsExecuteRecordServiceImpl extends SuperWhaleServiceImpl<AmsExecuteRecordMapper, AmsExecuteRecord> implements IAmsExecuteRecordService {
    @Resource
    private IAmsAirdropTaskService taskService;

    @Resource
    private IAmsAirdropProjectService projectService;

    @Override
    public PublicPage<ExecuteResponse> pageExecute(PageExecuteRequest executeRequest) {
        LambdaQueryWrapper<AmsExecuteRecord> wrapper = Wrappers.lambdaQuery();
        wrapper
                .eq(StrUtil.isNotBlank(executeRequest.getQueryValue()), AmsExecuteRecord::getExecuteId, executeRequest.getQueryValue())
                .or().like(StrUtil.isNotBlank(executeRequest.getQueryValue()), AmsExecuteRecord::getGroupName, executeRequest.getQueryValue())
                .orderByDesc(AmsExecuteRecord::getCreateTime);
        Page<AmsExecuteRecord> result = this.page(this.buildPageObj(executeRequest), wrapper);
        return this.convertPublicPage(result, s -> BeanUtil.copyProperties(s, ExecuteResponse.class));
    }

    @Override
    public void createExecute(CreateExecute createExecute) {
        AmsAirdropProject project = verifyProject(createExecute.getProjectId());
        AmsAirdropTask task = verifyTask(createExecute.getTaskId(), project.getAmsProjectId());
        AmsExecuteRecord amsExecute = BeanUtil.copyProperties(createExecute, AmsExecuteRecord.class);
        amsExecute.setUserId(SuperWhaleContext.getContext(PublicUserInfo.class).getUserId());
        amsExecute.setCreateTime(LocalDateTime.now());
        amsExecute.setTaskId(task.getAmsTaskId());
        this.save(amsExecute);
    }

    @Override
    public void updateExecute(UpdateExecute updateExecute) {
        AmsAirdropProject project = verifyProject(updateExecute.getProjectId());
        AmsAirdropTask task = verifyTask(updateExecute.getTaskId(), project.getAmsProjectId());
        AmsExecuteRecord execute = BeanUtil.copyProperties(updateExecute, AmsExecuteRecord.class);
        execute.setUserId(SuperWhaleContext.getContext(PublicUserInfo.class).getUserId());
        execute.setUpdateTime(LocalDateTime.now());
        execute.setTaskId(task.getAmsTaskId());
        this.updateById(execute);
    }

    /**
     * 验证项目id
     *
     * @param projectId 项目id
     * @return 项目实体
     */
    private AmsAirdropProject verifyProject(String projectId) {
        if (StrUtil.isBlank(projectId)) {
            throw new ErrorServiceException("projectId is required!");
        }
        AmsAirdropProject project = projectService.getById(projectId);
        if (project == null) {
            throw new ErrorServiceException("Airdrop Project Error!");
        }
        return project;
    }

    /**
     * 验证任务id
     *
     * @param taskId    任务id
     * @param projectId 项目id
     * @return 任务实体
     */
    private AmsAirdropTask verifyTask(String taskId, String projectId) {
        if (StrUtil.isBlank(taskId)) {
            throw new ErrorServiceException("taskId is required!");
        }
        LambdaQueryWrapper<AmsAirdropTask> wrapper = Wrappers.lambdaQuery();
        wrapper
                .eq(AmsAirdropTask::getAirdropProjectId, projectId)
                .eq(AmsAirdropTask::getAmsTaskId, taskId);
        AmsAirdropTask task = taskService.getOne(wrapper);
        if (task == null) {
            throw new ErrorServiceException("Task Error!");
        }
        return task;
    }
}
